;########################################################################
; J1d (Joust 1-dimensional)
; Single button contest submission
; 3.19.04
;
; All graphics and code produced by Troy Robinson (AKA Pongo)
; Please do not use these graphics without permission.
;
; Based (obviously) on the original arcade game Joust
;
; Update version .9a 3.20.2004
; -weakened flap strength to tighten up the controls a bit
; -some small collision related changes	
;########################################################################

Graphics 800,600;,0,2
SetBuffer BackBuffer()

SeedRnd MilliSecs()

Global gravity#=.15
Global maxfeathercount=20	; number of feathers to create on create_feathers() (slightly randomized)
Global enemycount=0			; number of active enemies
Global Gamestate=0 			; 0=menu, 1=playing, 2=Game Over, 3 quit
Global Gamelevel=0 			; set starting level
Global GameScore=0 			; set starting score
Global Highscore=0 			; set High score (load from file later)
Global Timer				; used to store millisec values
Global quit_timer
logo_ypos=-256				; main menu height position
frameTimer=CreateTimer(75) 	; timer used to lock the framerate for faster machines


Global player_dir=0 	; 0-right 3-left
Global player_xpos#=370	; set initial values for the player
Global player_ypos#=431	;
Global Player_xmov#=0	;
Global player_ymov#=0	;
Global player_frame=0	;
Global player_flap=0	; holds Millisecs value to calculate flaptime
Global flaptime=100		; amount of time for flap graphic to display

; load the player graphics
Global Player1=LoadAnimImage ("gfx\mount.png",72,72,0,6)
MaskImage player1,255,0,255

; load the background image
background=LoadImage("gfx\back.jpg")

;load the main title logo
logo=LoadImage("gfx\logo.png")
MaskImage logo,255,0,255

; load the game over graphic
gameover=LoadImage ("gfx\gameover.png")
MaskImage gameover,255,0,255

;load the collision masks. These are used to avoid "wingtip and tailfeather" collisions
Global collision=LoadAnimImage("gfx\collisionmask.png",72,72,0,6)
Global backgroundscroll=0

;load the feather graphics
Global feather_pic= LoadAnimImage ("gfx\feather.png",32,16,0,64) 
MaskImage feather_pic,255,0,255

; load the enemy graphics
Global enemy_pic= LoadAnimImage ("gfx\enemy.png",72,72,0,6)
MaskImage enemy_pic,255,0,255

Global flapsound=LoadSound("gfx\flap.wav") 
Global chirp=LoadSound("gfx\chirp1.wav")

;load and play the background sounds loop
Global backmusic=LoadSound ("gfx\ambient.wav")
LoopSound backmusic			; set the file to loop
SoundVolume backmusic,.2	; turn down the volume
PlaySound backmusic			; play the sound

Type Feather
	Field xpos#		; x position
	Field ypos# 	; Y position
	Field xmov#		; x movement
	Field ymov#		; y movement
	Field frame		; animation frame
	Field offset	; offset value for tumble effect
	Field mass		; Mass so feathers can fall at different rates
End Type 

Type enemy
	Field xpos#	; x position
	Field ypos#	; y position
	Field xmov#	; x movement
	Field ymov#	; y movement
	Field ylow
	Field frame	; animation frame
	Field flap
	Field dir	; direction
End Type 


;########################################################################
;########################################################################

Repeat ; mainloop

	While gamestate=0 ; menu
		If KeyHit(1) gamestate=3 ; fast exit when esc is pressed

		If KeyHit (57)	; spacebar
			gamestate=1				; initiate gameplay mode
			gamelevel=gamelevel+1	; increase to the next level
			create_enemy()			; create some bad guys!
			logo_ypos=-256			; reset logo position
			PlaySound flapsound
		EndIf	
		
		If MouseDown(1) createfeathers(MouseX(),MouseY()) ; bonus easter egg during title screen. Creates LOTS of feathers
		
		DrawBlock background,backgroundscroll ,0	; draw the background
		DrawBlock background,backgroundscroll-800,0	; background part 2
		
		logo_ypos=logo_ypos+5			; drop the main logo into place
		If logo_ypos>85 logo_ypos=85
			
		DrawImage logo,150,logo_ypos	; draw the main logo

		updatefeathers()	; update and draw all feathers

		Text 400,400,"Press space to begin play, or esc to exit",True
		Text 400,430,"To play, press space to flap. Highest competitor in a collision wins" ,True
		Text 550,570,"2004 Troy Robinson (Pongo)"
		
		Text 10,10,"Score:" + gamescore
		Text 650,10,"High Score:" + highscore
		
		
		WaitTimer (frametimer)	; delay the game for fast machines
				
		VWait() ; wait for monitor vertical sync
		Flip 0	
		
	Wend

	While gamestate=1 ; gameplay

		If KeyHit(1) gamestate=3 ; fast exit when esc is pressed
		
		If KeyHit(57); Or MouseHit(1); if spacebar is hit
		player_flap=MilliSecs()
		player_ymov=player_ymov+3
		PlaySound flapsound
		EndIf
			
		If player_flap + flaptime > MilliSecs()	player_frame=0 Else player_frame=1
				
		player_ypos=player_ypos-player_ymov
		player_ymov=player_ymov-gravity
			
		If player_ypos>430 Then ; if player hits the ground
		player_ypos=430			; do not allow them to drop further
		player_frame=2			; set frame to standing position
		player_ymov=-gravity	; stop jittering 
		backgroundscroll=backgroundscroll+1	; stop background scroll
		EndIf
		 
		If player_ypos<0 player_ymov=-4 ; bounce the player down if they hit the ceiling
		
		backgroundscroll=backgroundscroll-1			; scroll the background
		If backgroundscroll<0 backgroundscroll=800	; check limits
		DrawBlock background,backgroundscroll ,0	; draw back part 1
		DrawBlock background,backgroundscroll-800,0	; part 2

		DrawImage player1,player_xpos,player_ypos,player_frame+Player_dir ; draw player
			
		update_enemies() ; update and draw enemies,... check for collisions
		updatefeathers() ; update and draw all feathers

		If enemycount=0			; if all enemies are gone
		gamelevel=gamelevel+1	; next level
		create_enemy()			; new bad guys
		EndIf

		Text 10,10,"Score:" + gamescore
		Text 650,10,"High Score:" + highscore
		Text 400,10,"level:" + gamelevel
		

		WaitTimer (frametimer)	; delay the game for fast machines
				
		VWait() ; wait for monitor vertical sync
		Flip 0	
		
		;Cls ; not needed if background draws full screen
		
	Wend

	While gamestate=2 ; game over

		DrawBlock background,backgroundscroll ,0	; draw back part 1
		DrawBlock background,backgroundscroll-800,0	; part 2
	
		DrawImage gameover,270,200	; game over graphic
		
		If gamescore>highscore 
		Text 400,300,"Congratulations, You set a new highscore!",True
		EndIf 	
	
		Text 10,10,"Score:" + gamescore
		Text 650,10,"High Score:" + highscore
		Text 400,10,"level:" + gamelevel		
	
		update_enemies() ; update and draw enemies
		updatefeathers() ; update and draw all feathers
	
		If MilliSecs()-timer>2000 ;If 2 seconds has passed since the game finished
			Text 400,320,"press space to continue",True
			If KeyHit(57)
				If gamescore>highscore highscore=gamescore
				gamescore=0
				gamestate=0
				gamelevel=0
				PlaySound flapsound
				
				For a.enemy=Each enemy					; if any enemies are still flying
					Createfeathers(a\xpos+40,a\ypos+40)	; create a new feather explosion
					Delete a							; and delete them
					enemycount=enemycount-1				; decrease counter
				Next		
								
			EndIf  	
		EndIf 
	
		WaitTimer (frametimer)	; delay the game for fast machines
			
		VWait() ; wait for monitor vertical sync
		Flip 0	
		
	Wend
	
	FlushKeys()

Until gamestate=3 ; end main game loop

End ; end game

;########################################################################
;########################################################################

Function create_enemy() ; create new enemies at start of level
	For loop=1 To gamelevel
		enemycount=enemycount+1
		a.enemy=New enemy
		a\xpos=800
		a\ypos=Rnd(100,350)
		a\ylow=a\ypos +Rnd(50)
		a\xmov=Rnd(15,35)*.1
		If Rnd(1) Then a\xmov= -a\xmov 
		a\ymov=0
		If a\xmov<0 a\dir=3 Else a\dir=0
		a.enemy\frame=1
	Next
End Function 


Function Update_enemies()
	For a.enemy= Each enemy
		a.enemy\xpos=a.enemy\xpos+a.enemy\xmov ;add horizontal movement
		
		If a\ypos > a\ylow And a\frame=1; if enemy drops below "flap point" and is not already flapping
		a\flap=MilliSecs()				; grab the time	
		a\ymov=a\ymov+Rnd(3,6)			; give the flap a random strength
		EndIf
			
		If a\flap + flaptime > MilliSecs()	a\frame=0 Else a\frame=1	; check if flap has finished based on flaptime
				
		a\ypos=a\ypos-a\ymov		; add the y movement
		a\ymov=a\ymov-(gravity*.5)	; subtract gravity (a smaller amount of gravity makes the AI float more and flap less)
		
		If a\xpos>800 a.enemy\xpos=-72 ; if enemy is offscreen, Wrap around
		If a\xpos<-72 a.enemy\xpos=800 
		
		DrawImage enemy_pic,a\xpos,a\ypos,a\frame+a\dir ; draw the enemy
		
		;check for collisions
		If ImagesCollide (collision,a\xpos,a\ypos,a\frame,collision,player_xpos,player_ypos,player_frame+player_dir) ; if player intersects an enemy
			
			PlaySound chirp
			
			If player_ypos>a\ypos Then
				gamestate=2 ;die/game over
				createfeathers(player_xpos,player_ypos) ; run this 3 times to make a bigger explosion
				createfeathers(player_xpos,player_ypos)	; maybe replace with a better function
 				createfeathers(player_xpos,player_ypos)	; stating feather count as a variable
				player_ypos=600
				timer=MilliSecs ()	; grab current millisecs
				
			Else 
				Createfeathers(a\xpos+40,a\ypos+40) ;create a new feather explosion at the enemies last position
				Delete a							; delete the enemy
				enemycount=enemycount-1				; decrease the enemy count
				gamescore=gamescore+100				; add the score
				player_ymov=4						; bounce the player up a bit
				player_dir=3-player_dir 			; reverse direction (toggles between 3 and 0)
			EndIf 	
		EndIf 

	Next
End Function

Function Createfeathers(x,y) ; creates a feather explosion at position x,y
	feathercount=Rnd(maxfeathercount)+5
		For loop=1 To feathercount
		a.feather=New feather
		a\xpos=x
		a\ypos=y
		a\xmov=Rnd(-1,1)
		a\ymov=Rnd(1,3)
		a\offset=Rnd(-4,4)
		a\frame=Rnd(63)
		a\mass=(Rnd(10)*.01)
	Next
	
End Function

Function Updatefeathers()
	For a.feather=Each feather
		a\xpos=a\xpos+a\xmov+Sin(a\ypos)*a\offset;3
		a\ypos#=a\ypos#+a\ymov
		a\ymov=a\ymov+a\mass;+gravity#
		
		If a\xpos<0 Or a\xpos>800 Or a\ypos>600
			Delete a 
;		;If the particle has Not been deleted,... draw it on screen
		Else 
			DrawImage feather_pic,a\xpos,a\ypos,a\frame
			a\frame=a\frame+2
			If a\frame>63 a\frame=0
		EndIf
		
	Next
End Function